
#include <stdarg.h>
#include <util/delay.h>

#include "./apps.h"
#include "../menu/displayroutines.h"
#include "./realtimeclock.h"



void Beep(){
    PORTD |= (1 << Buzzer);
    _delay_ms(1);
    PORTD &=~(1 << Buzzer);
}


char* _2DigitValueWithLeadingNull(uint8_t *value){
    static char buffer[3];
    buffer[0] = '\0';
    char dummy[3];
    utoa( *value, dummy, 10);
    if (*value < 10){
        strcat(buffer, "0");
    }
    strcat(buffer, dummy);
    return buffer;
}

void DisplayTimeAt(uint8_t x,uint8_t y, t_datetime *time){
    PrintStringPos(x,y,"");

    PrintString(_2DigitValueWithLeadingNull(&time->hour));
    PrintChar(':');

    PrintString(_2DigitValueWithLeadingNull(&time->minute));
    PrintChar(':');

    PrintString(_2DigitValueWithLeadingNull(&time->second));

}

void DisplayDateAt(uint8_t x,uint8_t y, t_datetime *date, uint8_t withdayname){
    char zeitdigits[5];
    PrintStringPos(x,y,"");
    if(withdayname !=0){
        PrintString(date->dayname);
        PrintChar(' ');
    }
    PrintString(_2DigitValueWithLeadingNull(&date->day));

    PrintChar('.');
    PrintString(_2DigitValueWithLeadingNull(&date->month));

    PrintChar('.');
    utoa( date->year, zeitdigits, 10);
    PrintString(zeitdigits);
}

char* SwitchpointToASCII(const t_Switchpoint *aSwitchpoint){
    static char result[15];
    uint8_t i = 7;
    t_ranged_uint8_t index = {
         .value = FIRSTDAYOFWEEK,
         .top = 6,
         .bottom = 0
    };
    result[0] = '\0';
    result[14] = '\0';
    do{
        if(aSwitchpoint->days & (1 << index.value)){
            strncat(result, daynames[index.value],1);
        }
        else{
            strcat(result, "X");
        }
        rangedInc_uint8_t(&index);
    }while(--i);
    strcat(result, _2DigitValueWithLeadingNull((uint8_t *)&aSwitchpoint->hour));
    strcat(result, ":");
    strcat(result, _2DigitValueWithLeadingNull((uint8_t *)&aSwitchpoint->minute));
    result[12] = ' ';
    if( ((1<<FLAG_CHANNEL_HEIZUNG) & aSwitchpoint->flags) && ((1<<FLAG_CHANNEL_WARMWASSER) & aSwitchpoint->flags) ){
         result[12] = 'B';
    }
    else{
        if(aSwitchpoint->flags & (1<<FLAG_CHANNEL_HEIZUNG)){ result[12] = 'H';}
        if(aSwitchpoint->flags & (1<<FLAG_CHANNEL_WARMWASSER)){ result[12] = 'W';}
    }
    if(aSwitchpoint->flags & (1<<FLAG_ACTION)){
         result[13] = 'o';
    }
    else{
        result[13] = 'x';
    }
    if(! (aSwitchpoint->flags & (1<<FLAG_ISVALID))){
        result[13] = '-';
    }
    return result;
}
