
#include <stdint.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#include "./clocktimer.h"
#include "./apps.h"

#define SONNTAG 0
#define MONTAG 1
#define DIENSTAG 2
#define MITTWOCH 3
#define DONNERSTAG 4
#define FREITAG 5
#define SAMSTAG 6


t_Switchpoint eeSwitchpoints[SWITCHPOINTS_COUNT] EEMEM = {
    {
        .days = (1<<MONTAG)|(1<<DIENSTAG)|(1<<MITTWOCH)|(1<<DONNERSTAG)|(1<<FREITAG)|(1<<SAMSTAG)|(1<<SONNTAG),
        .hour = 5,
        .minute = 30,
        .flags = (1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_ACTION) | (1<<FLAG_ISVALID)
    },
    {
        .days = (1<<MONTAG)|(1<<DIENSTAG)|(1<<MITTWOCH)|(1<<DONNERSTAG)|(1<<FREITAG)|(1<<SAMSTAG)|(1<<SONNTAG),
        .hour = 5,
        .minute = 31,
        .flags = (1<<FLAG_CHANNEL_HEIZUNG) | (1<<FLAG_ACTION) | (1<<FLAG_ISVALID)
    },
    {
        .days = (1<<MONTAG)|(1<<DIENSTAG)|(1<<MITTWOCH)|(1<<DONNERSTAG)|(1<<FREITAG)|(1<<SAMSTAG)|(1<<SONNTAG),
        .hour = 5,
        .minute = 32,
        .flags = (1<<FLAG_CHANNEL_WARMWASSER) | (1<<FLAG_ACTION) | (1<<FLAG_ISVALID)
    },
    {
        .days = (1<<MONTAG)|(1<<DIENSTAG)|(1<<MITTWOCH)|(1<<DONNERSTAG)|(1<<FREITAG)|(1<<SAMSTAG)|(1<<SONNTAG),
        .hour = 5,
        .minute = 33,
        .flags = (1<<FLAG_CHANNEL_WARMWASSER) | (1<<FLAG_ACTION) | (1<<FLAG_ISVALID)
    },
};



void readSwitchpoint( uint8_t index, t_Switchpoint *destination){
    eeprom_read_block(destination, &eeSwitchpoints[index], sizeof(t_Switchpoint));
}


void writeSwitchpoint( uint8_t index,const t_Switchpoint *source){
    t_Switchpoint dummy;
    readSwitchpoint(index, &dummy);
    if(dummy.days != source->days){
        eeprom_write_byte((uint8_t*)&eeSwitchpoints[index].days, source->days);
    }
    if(dummy.hour != source->hour){
        eeprom_write_byte((uint8_t*)&eeSwitchpoints[index].hour, source->hour);
    }
    if(dummy.minute!= source->minute){
        eeprom_write_byte((uint8_t*)&eeSwitchpoints[index].minute, source->minute);
    }
    if(dummy.flags != source->flags){
        eeprom_write_byte((uint8_t*)&eeSwitchpoints[index].flags, source->flags);
    }
}

void deleteSwitchpoint(uint8_t index){
    t_Switchpoint dummy;
    readSwitchpoint(index, &dummy);
    if(dummy.flags & (1<<FLAG_ISVALID)){
        dummy.flags &= ~(1<<FLAG_ISVALID);
        eeprom_write_byte((uint8_t*)&eeSwitchpoints[index].flags, dummy.flags);
    }
    else{
        return;
    }
}


t_ClocktimerInfo Current[NUMBER_OF_USED_CHANNELS];
t_ClocktimerInfo Next[NUMBER_OF_USED_CHANNELS];

t_ranged_uint8_t OperatingMode = {
    .value = 0,
    .bottom = 0,
    .top = 3,
};

void updateChannels(t_datetime AktualTime)
{
    t_Switchpoint aSwitchpoint;
    int16_t switchminutes;
    uint8_t aktualdow = AktualTime.dow;
    int16_t aktualdiff;
    int16_t bestnextdiff[NUMBER_OF_USED_CHANNELS];
    uint8_t daymatch;
    cli();
    uint8_t i = NUMBER_OF_USED_CHANNELS - 1;
    do{
        bestnextdiff[i] = INT16_MIN;
    }while(i--);
    int16_t aktualminutes = ((AktualTime.hour * 60) + AktualTime.minute);
    sei();
    for (i = 0; i < 8; i++)
    {
        uint8_t j;
        for (j = 0; j < SWITCHPOINTS_COUNT; j++ )
        {
                readSwitchpoint(j, &aSwitchpoint);
                daymatch = ((aSwitchpoint.days & (1<<aktualdow))!=0);
                if (daymatch && (i == 0))
                {
                    //cli();
                    if ((aSwitchpoint.hour == AktualTime.hour) && (aSwitchpoint.minute == AktualTime.minute))
                    {

//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>Kanal einschalten<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<//
                        uint8_t channelcount;
                        for(channelcount = 0; channelcount < NUMBER_OF_USED_CHANNELS; channelcount++){
                            if((aSwitchpoint.flags & (1<<channelcount)) !=0){
                                //Beep();
                                Current[channelcount].value = aSwitchpoint;
                                Current[channelcount].valid_on_dow = aktualdow;
                            }
                        }
//>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>Kanal einschalten<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<//
                    }
                }
                if (daymatch)
                {
                    cli();
                    switchminutes = (i * 1440) + (aSwitchpoint.hour * 60) + aSwitchpoint.minute;
                    aktualdiff = aktualminutes  - switchminutes;
                    sei();
                    cli();
                    uint8_t channelcount;
                    for(channelcount = 0; channelcount < NUMBER_OF_USED_CHANNELS; channelcount++){
                        if ((aktualdiff < 0) && (aktualdiff > bestnextdiff[channelcount]))
                        {
                            if((aSwitchpoint.flags & (1<<channelcount)) !=0){
                                bestnextdiff[channelcount] = aktualdiff;
                                Next[channelcount].value = aSwitchpoint;
                                Next[channelcount].valid_on_dow = aktualdow;
                            }
                        }
                    }
                    sei();
                }
        }
        aktualdow += 1;
        if (aktualdow == 7) //6 Index of last Day
        {
            aktualdow = 0;
        }
    }
    sei();
}
