#ifndef CLOCKTIMER_H_INCLUDED
#define CLOCKTIMER_H_INCLUDED

#include "avr/eeprom.h"

#include "../types/mytypes.h"
#include "../types/ranged.h"

#define SONNTAG 0
#define MONTAG 1
#define DIENSTAG 2
#define MITTWOCH 3
#define DONNERSTAG 4
#define FREITAG 5
#define SAMSTAG 6

#define FLAG_CHANNEL_HEIZUNG 0
#define FLAG_CHANNEL_WARMWASSER 1
#define FLAG_CHANNEL_3 2
#define FLAG_CHANNEL_4 3
#define FLAG_CHANNEL_5 4
#define FLAG_CHANNEL_6 5
#define FLAG_ACTION 6
#define FLAG_ISVALID 7

/*OperatingMode*/
#define off    0 //Heizung aus, Warmwasser aus
#define spring 1 //Heizung an (Nachtabsenkung aus), Warmwasser an
#define summer 2 //Heizung aus, Warmwasser an
#define winter 3 //Heizung an (Nachtabsenkung aktiv), Warmwasser an

extern t_ranged_uint8_t OperatingMode;

typedef struct s_Switchpoint {
    uint8_t days;      //Bit0 Sonntag, Bit1 Montag, Bit2 Die..
    uint8_t hour;
    uint8_t minute;
    uint8_t flags;     //Bit0 .. Bit5 Channels -> nur 2 Kanle benutzt Kanal0 Heizung, Kanal1 Warmwasser
                       //Bit6 Action -> 0 Channel OFF, 1 Channel ON
                       //Bit7 entry valid -> 1 entry valid, 0 entry invalid
}t_Switchpoint;
//128 Schaltpunktdefinitionen sind mglich
//alle Kanle mssen sich diese Definitionen teilen
//jeder Schaltpunkt kann mehrere Kanle schalten
//es werden nur die Kanle Heizung und Warmwasser untersttzt
//eine einfache Erweiterung ist mglich

//#define BYTES_IN_EEPROM 512
//#define SIZE_OF_T_SWITCHPOINT sizeof(t_switchpoint)
#define SWITCHPOINTS_COUNT 128
//es werden nur Schaltpunkt1 Heizung und Schaltpunkt2 Warmwasser untersttzt
#define NUMBER_OF_USED_CHANNELS 2
extern t_Switchpoint eeSwitchpoints[SWITCHPOINTS_COUNT] EEMEM;

typedef struct s_ClocktimerInfo{
    t_Switchpoint value;
    uint8_t valid_on_dow;
} t_ClocktimerInfo;


extern t_ClocktimerInfo Current[NUMBER_OF_USED_CHANNELS];
extern t_ClocktimerInfo Next[NUMBER_OF_USED_CHANNELS];


void readSwitchpoint( uint8_t index, t_Switchpoint *destination);
void writeSwitchpoint( uint8_t index,const t_Switchpoint *source);
void deleteSwitchpoint(uint8_t index);

void updateChannels(t_datetime datetime);
#endif // CLOCKTIMER_H_INCLUDED
